/*
	SANFR8IN.H - Frame Relay Device Driver (S508) User Interface Header
	Copyright (c) 1996 Sangoma Technologies Inc.
	v1.00 April/96

	Set tabs to 4 spaces when viewing.
*/

//
// DeviceIoControl command codes. Must include "winioctl.h" in app for
// definition of CTL_CODE and defines.
//

// Standard CBLOCK-passing command:
// Input: ptr to FRAME_RELAY_CBLOCK structure
// Output: ptr to FRAME_RELAY_CBLOCK structure (may be same as input)
// Uses Buffered I/O. Synchronous call.
#define IoctlCodeCblockCommand	CTL_CODE(FILE_DEVICE_UNKNOWN, 10, METHOD_BUFFERED, FILE_ANY_ACCESS)

// Command to send a frame:
// Input: ptr to FRAME_RELAY_CBLOCK structure
// Output: ptr to FRAME_RELAY_CBLOCK structure (may be same as input)
// Uses Direct I/O (Irp->MdlAddress). Synchronous call.
// Note: Frame only queued for transmission when ReturnCode is 0x00. If not,
//       must repeat this command with reinitialized structure.
//       The Command member need not be initialized.
#define IoctlCodeWriteCommand	CTL_CODE(FILE_DEVICE_UNKNOWN, 11, METHOD_OUT_DIRECT, FILE_ANY_ACCESS)

// Command to read a frame:
// Input: NULL
// Output: ptr to FRAME_RELAY_READ_INFO (plus adequate data storage area)
// Uses Direct I/O (Irp->MdlAddress). Blocking call on device handle.
// Note: Since this command blocks all further I/O requests, it must be done
//       on a separate device handle. Just open the same device again and use
//       this handle for reading.
#define IoctlCodeReadCommand	CTL_CODE(FILE_DEVICE_UNKNOWN, 12, METHOD_IN_DIRECT, FILE_ANY_ACCESS)

// Command to retrieve status information:
// Input: NULL
// Output: ptr to FRAME_RELAY_STATUS_INFO structure
// Uses Buffered I/O. Synchronous call.
#define IoctlCodeStatusCommand	CTL_CODE(FILE_DEVICE_UNKNOWN, 13, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// ===== MAILBOX COMMANDS =====
// For use with IoctlCodeCblockCommand.
//
#define INFORMATION_WRITE       0x01	// not used directly
#define	ISSUE_SIGNALLING_FRAME	0x03
#define SET_DLCI_CONFIGURATION  0x10
#define	READ_DLCI_CONFIGURATION	0x11
#define DISABLE_COMMUNICATIONS  0x12	// also terminates IoctlCodeReadCommand if pending
#define ENABLE_COMMUNICATIONS   0x13
#define READ_DLC_STATUS         0x14
#define READ_DLC_STATISTICS     0x15
#define	FLUSH_DLC_STATISTICS	0x16
#define	LIST_ACTIVE_DLCI		0x17
#define	FLUSH_INFO_BUFFERS		0x18	// not to be used
#define ADD_DLCI				0x20
#define	DELETE_DLCI				0x21
#define ACTIVATE_DLCI			0x22
#define	DEACTIVATE_DLCI			0x23
#define	READ_MODEM_STATUS		0x30
#define	SET_MODEM_STATUS		0x31
#define	READ_COMMS_ERR_STATS	0x32
#define FLUSH_COMMS_ERR_STATS   0x33
#define READ_CODE_VERSION       0x40
#define DISCARD_FRAMES          0x41
#define SET_INTERRUPT_TRIGGERS  0x50	// not to be used
#define READ_INTERRUPT_TRIGGERS 0x51

//
// Driver-defined return codes for CBLOCK-associated commands.
//
#define RCODE_TIMEOUT_ERROR		0x7f	// Card is not responding to command.
#define RCODE_INIT_ERROR		0x7e	// Card could not be initialized.
										// (SET_DLCI_CONFIGURATION only)

//
// All structures below must be packed on 1-byte boundaries.
//

#pragma pack(1)

//
// Control block structure used for IoctlCodeCblockCommand and
// IoctlCodeWriteCommand, as input and output data.
//
typedef struct _FRAME_RELAY_CBLOCK {
	UCHAR OpFlag;			// internal use only
	UCHAR Command;			// frame relay command code
	USHORT BufferLength;	// length of data associated with command
	UCHAR ReturnCode;		// return code
	USHORT DLCI;			// DLCI
	UCHAR FecnBecnDeCr;		// various frame-relevant bits
	UCHAR reserved[8];
	UCHAR Data[1];			// variable-length array containing data
} FRAME_RELAY_CBLOCK, *PFRAME_RELAY_CBLOCK;

//
// May be used as an aid for filling in the data area of the
// SET_DLCI_CONFIGURATION (DLCI=0) command. See printed manual.
//
typedef struct _FRAME_RELAY_CONFIG {
	USHORT Station;
	USHORT ConfigBits;
	USHORT BaudRateKbps;
	USHORT ClockingInterface;
	USHORT MaxIFrameLength;
	USHORT T391;
	USHORT T392;
	USHORT N391;
	USHORT N392;
	USHORT N393;
	USHORT CIRfwd;
	USHORT Bcfwd;
	USHORT Befwd;
	USHORT CIRbwd;
	USHORT Bcbwd;
	USHORT Bebwd;
	USHORT DLCI[1];
} FRAME_RELAY_CONFIG, *PFRAME_RELAY_CONFIG;

//
// Information returned by IoctlCodeStatusCommand.
// See printed manual section on interface area bytes.
//
typedef struct _FRAME_RELAY_STATUS_INFO {
	UCHAR BasePortRegister;			// contents of base port register
	UCHAR ExceptionConditionByte;	// contents of exception condition byte
	UCHAR ModemStatusByte;			// contents of modem status byte
} FRAME_RELAY_STATUS_INFO, *PFRAME_RELAY_STATUS_INFO;

//
// Equivalent to Receive Status Element in Manual.
// Data of variable length follows.
//
typedef struct _FRAME_RELAY_READ_INFO {
	UCHAR Status;					// set to 0x01 on return if frame read
	USHORT FrameLength;				// set to length of frame that follows
	USHORT DLCI;					// DLCI that data arrived on
	UCHAR FecnBecnDeCr;				// various frame-related bits
	UCHAR Reserved[10];
	UCHAR Data[1];					// variable-length frame data
} FRAME_RELAY_READ_INFO, *PFRAME_RELAY_READ_INFO;

#pragma pack()


